//for debug
#include <iostream>

#include <sstream>

#include <qlabel.h>
#include <qmessagebox.h>

#include <utils/str.hh>

#include <qtlook/dialog.hh>
#include <qtlook/button.hh>
#include <qtlook/listbox.hh>
#include <qtlook/line_edit.hh>

#include <pair_ops.hh>

using namespace qtlook;

namespace
{
    void fill_listbox(QListBox *lb,const std::string& items)
    {
	std::istringstream ist(items);
	std::string item;
	while(std::getline(ist,item))
	{
	    item = alt::trim(item);
	    if (item.empty()) continue;
	    lb->insertItem(item.c_str());
	}
    }
}


qtlook::dialog::dialog()
{
    lstack_.push(new QVBoxLayout(&main_widget_));
}

long qtlook::dialog::run()
{
    main_widget_.exec();
    command_map::const_iterator it = commands_.find(main_widget_.lastaction_);
    return (it != commands_.end())?it->second:(-1);
}

void qtlook::dialog::add_button(const std::string& id, const std::string& name, long key)
{
    std::cerr<<"add_button: id="<<id<<" key="<<key<<std::endl;
    qtlook::button *bt = new qtlook::button(id,name,&main_widget_);
    lstack_.top()->addWidget(bt,0,Qt::AlignLeft);
    
    widgets_[id] = bt;
    commands_[id]= key;
    types_[id] = ::dialog::button;
    
    QObject::connect(bt,SIGNAL(clicked(const std::string&)),
                     &main_widget_,SLOT(intercept(const std::string&)));
    
}


void qtlook::dialog::add_listbox(const std::string& id,const std::string& items,long key)
{
    std::cerr<<"add_listbox: id="<<id<<" key="<<key<<std::endl;
    qtlook::listbox *lb = new qtlook::listbox(id,&main_widget_);
    lstack_.top()->addWidget(lb,0,Qt::AlignLeft);

    fill_listbox(lb,items);
    
    widgets_[id] = lb;
    commands_[id]= key;
    types_[id] = ::dialog::listbox;
    
    QObject::connect(lb,SIGNAL(selected(const std::string&)),
                     &main_widget_,SLOT(intercept(const std::string&)));
}


void qtlook::dialog::add_textbox(const std::string& id,const std::string& txt,long key)
{
    std::cerr<<"add_textbox: id="<<id<<" key="<<key<<std::endl;
    qtlook::line_edit *led = new qtlook::line_edit(id,txt,&main_widget_);
    lstack_.top()->addWidget(led,0,Qt::AlignLeft);

    widgets_[id] = led;
    commands_[id]= key;
    types_[id] = ::dialog::textbox;
    
    QObject::connect(led,SIGNAL(changed(const std::string&)),
                     &main_widget_,SLOT(intercept(const std::string&)));
}

void qtlook::dialog::add_label(const std::string& id,const std::string& name)
{
    std::cerr<<"add_label: id="<<id<<" name="<<name<<std::endl;
    QLabel *lb = new QLabel(name,&main_widget_);
    lstack_.top()->addWidget(lb,0,Qt::AlignLeft);

    widgets_[id] = lb;
    types_[id] = ::dialog::label;
}

void qtlook::dialog::add_variable(const std::string& id,const std::string& value)
{
    std::cerr<<"add_variable: id="<<id<<" value="<<value<<std::endl;
    vars_[id] = value;
    types_[id] = ::dialog::variable;
}


void qtlook::dialog::begin_hbox()
{
    std::cerr<<"begin_hbox:"<<std::endl;
    QHBoxLayout *l = new QHBoxLayout(lstack_.top());
    lstack_.push(l);
}

void qtlook::dialog::end_hbox()
{
    std::cerr<<"end_hbox:"<<std::endl;
    lstack_.pop();
}

void qtlook::dialog::begin_vbox()
{
    std::cerr<<"begin_vbox:"<<std::endl;
    QVBoxLayout *l = new QVBoxLayout(lstack_.top());
    lstack_.push(l);
}

void qtlook::dialog::end_vbox()
{
    std::cerr<<"end_vbox:"<<std::endl;
    lstack_.pop();
}



void qtlook::dialog::set_attr(const std::string& id,const std::string& name,const std::string& value)
{
    type_map::const_iterator t = types_.find(id);
    widget_map::const_iterator w = widgets_.find(id);
    
    if ((t == types_.end()) || 
       (w == widgets_.end() && t->second != ::dialog::variable)) return;

    if (::dialog::label == t->second)
    {
	    std::cerr<<"setting new value to label"<<std::endl;
	    if ("value" == name) static_cast<QLabel*>(w->second)->setText(value.c_str());
    }
    if (::dialog::button == t->second)
    {
	    std::cerr<<"setting new value to label"<<std::endl;
	    if ("label" == name) static_cast<QPushButton*>(w->second)->setText(value.c_str());
    }
    else if (::dialog::listbox == t->second)
    {
	    std::cerr<<"setting new value to listbox"<<std::endl;
	    if ("value" == name) fill_listbox(static_cast<QListBox*>(w->second),value);
    }
    else if (::dialog::textbox == t->second)
    {
	    std::cerr<<"setting new value to textbox"<<std::endl;
	    if ("value" == name) static_cast<QLineEdit*>(w->second)->setText(value.c_str());
	    if ("type" == name) static_cast<QLineEdit*>(w->second)->setEchoMode(
	                        value == "password"?QLineEdit::Password : QLineEdit::Normal);
    }
    else if (::dialog::variable == t->second)
    {
	    std::cerr<<"setting new value to variable"<<std::endl;
	    if ("value" == name) vars_[id]=value;
    }

}

std::string qtlook::dialog::get_attr(const std::string& id,const std::string& name)
{
    type_map::const_iterator t = types_.find(id);
    widget_map::const_iterator w = widgets_.find(id);
    if ((t == types_.end())
    || (w == widgets_.end() && t->second != ::dialog::variable)) return "";

    if (::dialog::label == t->second)
    {
	    std::cerr<<"getting value from label"<<std::endl;
	    if ("value" == name) return static_cast<QLabel*>(w->second)->text();
    }
    else if (::dialog::listbox == t->second)
    {
	    std::cerr<<"getting value from listbox"<<std::endl;
	    if ("current" == name) return static_cast<QListBox*>(w->second)->currentText();
    }
    else if (::dialog::textbox == t->second)
    {
	    std::cerr<<"getting value from listbox"<<std::endl;
	    if ("value" == name) return static_cast<QLineEdit*>(w->second)->text();
    }
    else if (::dialog::variable == t->second)
    {
	    std::cerr<<"getting value from listbox"<<std::endl;
	    if ("value" == name) return vars_[id];
    }

    return "";
}

void alert(const std::string& caption,const std::string& message)
{
    QMessageBox::critical(0,caption.c_str(),message.c_str());
}
