#include <qtlook/look.hh>

#include <utils/str.hh>

#include <bus_io.hh>
#include <command_io.hh>

#include <iostream>
#include <sstream>

namespace
{
    struct evaluate
    {
	evaluate(command& c,evaluator& e): c_(c), e_(e) {}
	void operator()(const std::string& name,const std::string& value)
	{
	    c_(name,e_(value));
	}
	command&   c_;
	evaluator& e_;
    };
}


/** load dialog from the xml description, also create appropriate command sets */
static void
look_file(const command& cmd, dialog_ptr& d, command_set& c,evaluator& e)
{
	qtlook::dialog *new_d = new qtlook::dialog();
	c = load_dialog(cmd("name"), *new_d,e);
	d = new_d;
}

/** load url using mapper */
static command
look_view(const command& cmd, const mapper& m)
{
	try
	{
		return m.map(cmd("id"));
	} catch(const mapper::missing&)
	{
		std::cerr << "no such id: " << cmd("id") << std::endl;
	}
	return command("/ctrl/look")("action", "quit");
}

static void
look_attr(const command& cmd, dialog_ptr& d)
{
    std::cerr<<"look_attr was called\n";
    if (d.get()) d->set_attr(cmd("ctrlid"),cmd("field"),cmd("value"));
}

bool
process_cmd(const command& cmd, command_set& c)
{

	command ecmd = cmd;
	ecmd.for_each(evaluate(ecmd,qtlook_eval::instance()));
	ecmd.name(qtlook_eval::instance()(cmd.name()));//also evaluate command name

	std::cerr<<"processing command:"<<ecmd.name()<<std::endl;

	if("/ctrl/look" == ecmd.name())
	{// process itself
		if("file" == ecmd("action"))
			look_file(ecmd, qtlook_dialog::instance(), c, qtlook_eval::instance());
		else if("view" == ecmd("action"))
			return process_cmd(look_view(ecmd, *qtlook_mapper),c);
		else if("attr" == ecmd("action"))
			look_attr(ecmd, qtlook_dialog::instance());
		else if("quit" == ecmd("action"))
			return false;
	}
	else
	{//ask chooser
		//send request
		std::ostringstream scmd;
		scmd<<ecmd;
		send(">>",scmd.str(),std::cin,std::cout,std::cerr);
		
		//read answer: 
		std::ostringstream answer;
		while(std::cin)
		{
		    std::string direction;
		    std::getline(std::cin,direction);
		    if ("<<" == direction)
		    {
			std::istringstream ist(receive(std::cin,std::cout,std::cerr));
			command cmd;
			while (ist>>cmd)
			  if (alt::starts_with(cmd.name(),"/error")) cmd_alert(cmd);
			break;
		    }
		    else
			std::cout<<"<>"<<std::endl;
		}
	}
	return true;
}
